use std::{fs, collections::HashSet};

use anyhow::Result;
use itertools::Itertools;


fn main() -> Result<()> {
    let input = fs::read_to_string("input/03.txt")?;

    let lines: Vec<&str> = input
        .split("\n")
        .filter(|s| *s != "")
        .collect::<Vec<&str>>();

    let groups = lines
        .into_iter()
        .map(|s| s.chars().collect::<HashSet<char>>())
        .chunks(3);

    let badges = groups
        .into_iter()
        .map(|g| g.reduce(|acc, e| acc.intersection(&e).cloned().collect()).unwrap())
        .map(|g| g.into_iter().next().unwrap())
        .collect_vec();

    let result: i32 = badges
        .into_iter()
        .map(char_to_priority)
        .sum();

    println!("{:?}", result);
    
    return Ok(());
}

fn char_to_priority(c: char) -> i32 {
    if c.is_uppercase() {
        return c as i32 - 65 + 27;
    } else {
        return c as i32 - 96;
    }
}

