use std::fs;

use anyhow::Result;
use itertools::Itertools;


fn main() -> Result<()> {
    let input = fs::read_to_string("input/05.txt")?;

    let mut lines = input
        .split("\n")
        .into_iter();

    //         [F] [Q]         [Q]        
    // [B]     [Q] [V] [D]     [S]        
    // [S] [P] [T] [R] [M]     [D]        
    // [J] [V] [W] [M] [F]     [J]     [J]
    // [Z] [G] [S] [W] [N] [D] [R]     [T]
    // [V] [M] [B] [G] [S] [C] [T] [V] [S]
    // [D] [S] [L] [J] [L] [G] [G] [F] [R]
    // [G] [Z] [C] [H] [C] [R] [H] [P] [D]
    //  1   2   3   4   5   6   7   8   9 
    //
    // move 3 from 5 to 2
    
    let to_strip: &[_] = &[' ', '[', ']'];
    
    let initial_data = lines
        .by_ref()
        .take_while(|s| *s != "")
        .map(|s| s.chars().collect_vec())
        .map(|c| c.chunks(4).map(|x| x.to_owned()).collect_vec())
        .map(|l| l
            .into_iter()
            .map(|x| x.into_iter().collect::<String>())
            .collect_vec())
        .map(|l| l.into_iter().map(|x| x.trim_matches(to_strip).to_owned()).collect_vec())
        .take_while(|v| v[0] != "1")
        .collect_vec();
    
    let initial_data = transpose(&initial_data);

    let stacks = initial_data.into_iter()
        .map(|mut l| {l.reverse(); l})
        .map(|l| l.into_iter().take_while(|s| *s != "").collect_vec())
        .collect_vec();

    let moves = lines
        .dropping(1)
        .filter(|s| *s != "")
        .map(|s| s.split(" ").collect_vec())
        .map(|p| (p[1].parse::<usize>().unwrap(), p[3].parse::<usize>().unwrap(), p[5].parse::<usize>().unwrap()))
        .collect_vec();

    let final_state = moves
        .into_iter()
        .fold(stacks, execute_move);

    print_state(&final_state);

    let answer = final_state
        .into_iter()
        .map(|mut l| l.pop().unwrap())
        .join("");

    println!("{answer}");

    return Ok(());
}


fn execute_move(mut stacks: Vec<Vec<String>>, (count, from, to): (usize, usize, usize)) -> Vec<Vec<String>> {
    // print_state(&stacks);
    // println!("move {count} from {from} to {to}");
    let mut values = (0..count).map(|_| stacks[from - 1].pop().unwrap()).collect_vec();
    values.reverse();
    values.into_iter().for_each(|v| stacks[to - 1].push(v));
    
    return stacks;
}


fn print_state(stacks: &Vec<Vec<String>>) {
    let lines = (0..(stacks.iter().map(|v| v.len()).max().unwrap()))
        .map(|i| stacks
            .iter()
            .map(|inner| {if inner.len() > i {inner[i].clone()} else {" ".to_owned()}})
            .collect_vec())
        .collect_vec();
    
    lines
        .into_iter()
        .map(|l| l.into_iter().map(|s| {if s == "" {" ".to_owned()} else {s}}).collect_vec())
        .for_each(|l| println!("{:?}", l));
}


fn transpose<T>(v: &Vec<Vec<T>>) -> Vec<Vec<T>>
where
    T: Clone,
{
    assert!(!v.is_empty());
    (0..v[0].len())
        .map(|i| v.iter().map(|inner| inner[i].clone()).collect::<Vec<T>>())
        .collect()
}
